#!/usr/bin/env python
'''
** This file consitutes part of NCSSWiki
 * Version 2.0 (beta release 2)
 * (c) 2006 Chris Neugebauer -- http://noogz.net -- chris@noogz.net
 * and contributors from the 2006 National Computer Science School
 *
 * NCSSWiki is the legal property of Christopher Neugebauer and contributors,
 * and is free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation; either version 2 of the License, or (at your option) any later 
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * or visit http://www.gnu.org/licenses/gpl.html
 *'''
## wikiwikiwiki parser

## Rev 1: Add links parser.

## To Do:
## 1) Imlement escaped-hex into linking code (fun fun fun)

import re
import config
conf = config.py

## Global Vars
wiki_base = "article.cgi?article=" # Base location for wiki.


	

def link_colour(match):
	## Is returned name of article, checks database and changes link colour
	## match is a regex group.
	global conf
	
	article = match.group(1) #get the article title
	import wiki_getdb

	if wiki_getdb.exists(article):
		colour=conf["valid_links"] # Colour for existent article
	else:
		colour=conf["invalid_links"] # Colour for non-existent article
	
	return  "<a style=\"color: "+colour+"\" href=\"" + wiki_base + article + "\">"+article+"</a>"


def parse_links(string_to_parse):
	## Parses internal links.
	## Do not call before parse_headings

	re_wlink	 =   r"\[\[(.*?)\]\]"		# Matches WikiLink [[link]]

	## Compiled expressions
	com_re_wlink = re.compile(re_wlink,re.MULTILINE)

	## The Ind.
	temp	=	re.sub(com_re_wlink,link_colour,string_to_parse)

	re_wilink	   =   r"\[(.*?)\]"			# Matches internal WikiLink
	re_hilink	   =   "<a href=\"\\1\">\\1</a>"

	com_re_wilink   = re.compile(re_wilink,re.MULTILINE)

	return re.sub(com_re_wilink,re_hilink,temp)


def parse_headings(string_to_parse):
	## Parses headings, beginning with 5th-level, through to 1st.
	## ===foo=== is a 3rd level heading.

	for i in range(5,0,-1):
		re_whead	=   "^" + (r"="*i)+r"(.*?)"+(r"="*i)
		re_hhead	=   "<h"+str(i)+">\\1</h"+str(i)+">"

		com_re_whead	=   re.compile(re_whead,re.MULTILINE)
		string_to_parse = re.sub(com_re_whead,re_hhead,string_to_parse)

	return string_to_parse


def parse_paragraphs(string_to_parse):
	## Parse paragraphs.  Could be fun.
	## It now also converts stray \n's to <br />

	# Replace double-newline with begin/end paragraph.
	re_wpar		 =   (r"\n\n")
	re_hpar		 =   ("</p>bork<p>") #I've added a generic here for later fixing.
	com_re_wpar	 =   re.compile(re_wpar,re.MULTILINE)
	
	# Deal with double-newlines, but remove all whitespace from beginning and end
	string_to_parse =   re.sub(com_re_wpar,re_hpar,string_to_parse.strip())

	# Add breaks to any newline that does not end with a header or list tag
	# Avoids adding redundant newlines.
	re_wbr		  =   (r"(?<!(</h[1-5]>|</li>).)\n")
	re_hbr		  =   ("<br />\n")
	com_re_wbr	  =   re.compile(re_wbr,re.MULTILINE)

	temp	          =   re.sub(com_re_wbr,re_hbr,string_to_parse)

	#Remove the word 'bork' and replace with newline
	re_wpar		 =   ("</p>bork<p>")
	re_hpar		 =   ("</p>\n<p>")
	com_re_wpar	 =   re.compile(re_wpar,re.MULTILINE)
	return		  re.sub(com_re_wpar,re_hpar,temp)


def parse_formatting(string_to_parse):
	## Does bold, italics, underline etc
	## ''''foo'''' is underline, '''foo''' is bold, ''foo'' is italic

	# Do 4 ticks, then 3
	for i in range(4,1,-1):
		re_wform	=   (r"'"*i) + r"(.*?)" +(r"'"*i)

		# Choose HTML equivalent tag based on number of ticks being matched.
		if i==4: # 4 ticks underline
			tag = "u"
		elif i==3: # 3 ticks bold
			tag = "strong"
		elif i==2: # 2 ticks italic
			tag = "em"

		re_hform	=   ("<"+tag+">"+"\\1</"+tag+">")
		com_re_wform	=   re.compile(re_wform,re.MULTILINE)
		string_to_parse =   re.sub(com_re_wform,re_hform,string_to_parse) 

	return string_to_parse

def parse_lists(string_to_parse):
	## Matches lists
	## Match * as bullet point.
	
	re_wbullet	  =   r"^\*(.*)$"
	re_hbullet	  =   ("<li>\\1</li>")

	com_re_wbullet  =   re.compile(re_wbullet,re.MULTILINE)

	return re.sub(com_re_wbullet,re_hbullet,string_to_parse) 

def parse_wiki(string_to_parse):
	avar = 0 #Counter variable.  Deals with python's dodgy for loops
	temp = "" #Stores the parsed contents
	for i in re.split(re.compile(r"<nowiki>(.*?)</nowiki>",re.DOTALL),string_to_parse):
		i = re.sub("\\r","",i)
		if avar % 2 == 0: # Even-numbered blocks are parsed
			temp += parse_paragraphs( #This leveling code is extemely scientific.
					parse_lists(	 #Will stuff up if not done in this order.  
						parse_formatting(
							parse_links(
								parse_headings(i)))))
		else:
			temp += i
		avar += 1
	return temp
